/////////////////////////////////////////////////////////////////////////////
//
//     MContinueList
//
//  Apply static cuts into a tasklist as a list of MContinue. Rules for 
//  MContinue list can be read from enviroment card (through TEnv) or can 
//  be added manually by AddCut(). Actually this class can be used for
//  a generic MContinue list.
//
/////////////////////////////////////////////////////////////////////////////
#include <TOrdCollection.h>
#include <TEnv.h>
#include <TClass.h>

#include "MLog.h"
#include "MLogManip.h"
#include "MParList.h"
#include "MTaskList.h"
#include "MF.h"
#include "MFilterList.h"
#include "MContinue.h"
#include "MContinueList.h"

ClassImp(MContinueList);

using namespace std;

// --------------------------------------------------------------------------
//
// Default constructor
//
MContinueList::MContinueList(const char *name, const char *title)
    : fTaskList(0), fEnv(0)
{
    fName  = name  ? name  : "MContinueList";
    fTitle = title ? title : "List of MContinue";

    //SetBit(kIsOwner);

    fFilters = new MFilterList("||","MFilterList","Filter list of MContinueList");
    //fFilters->AddToList(new MF("0>1"));
    SetFilter(fFilters);
}


// --------------------------------------------------------------------------
//
// Copy constructor
//
MContinueList::MContinueList(MContinueList *cont)
{
    fName  = cont->GetName();
    fTitle = cont->GetTitle();

    fFilters = (MFilterList *)cont->GetFilterList()->Clone();
    SetFilter(fFilters);
}


// --------------------------------------------------------------------------
//
// Destructor
//
MContinueList::~MContinueList()
{
    if (TestBit(kIsOwner))
        delete GetFilter();
}



Int_t MContinueList::AddToList(MContinue *cont)
{
    if (!cont)
        return kFALSE;
    //
    // Get Name of new container
    //
    const char *name = cont->GetName();

    //
    // Check if the new container is already existing in the list
    //
    if (fFilters->FindObject(name))
    {
	*fLog << warn << dbginf << "Warning: Container '" << cont->GetName() << ", 0x" << (void*)cont;
	*fLog << "' already existing in '" << GetName() << "'... ignoring." << endl;
	return kTRUE;
    }
     
    *fLog << inf << "Adding " << name << " to " << GetName() << "... " << flush;

    fFilters->AddToList(cont->GetFilter());

    *fLog << "Done." << endl;

    return kTRUE;
}


// ----------------------------------------------------------------------------
//
// PreProcess
//
/*
Int_t MContinueList::PreProcess(MParList *plist)
{
    ResetBit(kFilterIsPrivate);

    if (!GetFilter())
    {
        if (IsAllowEmpty())
        {
            *fLog << warn << GetDescriptor() << " - WARNING: Empty filter list found... task removed." << endl;
            return kSKIP;
        }

        *fLog << err << dbginf << "Unknown fatal Error! (fFilter=NULL?!?)" << endl;
        return kFALSE;
    }

    fTaskList = (MTaskList*)plist->FindTaskListWithTask(this);
    if (!fTaskList)
    {
        *fLog << err << dbginf << "ERROR - MTasklist not found... abort." << endl;
        return kFALSE;
    }

    if (fTaskList->FindObject(GetFilter()))
    {
        *fLog << inf << dbginf << "The filter list is already in the tasklist..." << endl;
        return kTRUE;
    }

    if (!fTaskList->AddToListBefore(GetFilter(), this))
    {
        *fLog << err << dbginf << "ERROR - Adding filter before MContinue... abort." << endl;
        return kFALSE;
    }

    // Make sure, that everything is correctly propageted to the childs
    GetFilter()->SetDisplay(fDisplay);
    GetFilter()->SetLogStream(fLog);

    // Remeber that the filter is not already in tasklist
    SetBit(kFilterIsPrivate);

    return GetFilter()->CallPreProcess(plist);
}


// ----------------------------------------------------------------------------
//
// Process
//
/*Int_t MContinueList::Process()
{
    if(!fFilters->CallProcess()){
	*fLog << err << "Error - Processing " << fFilters->GetName() << " in " << fName << endl;
	return kFALSE;
    }
    
    return kTRUE;
}
*/

// ------------------------------------------------------------------------------
//
// PostProcess
//
/*
Int_t MContinueList::PostProcess()
{
    if (!TestBit(kFilterIsPrivate))
        return kTRUE;

    if (fTaskList->RemoveFromList(fFilters))
        return kTRUE;

    *fLog << err << "ERROR: MContinueList::PostProcess - Cannot remove FilterList from tasklist" << endl;

    return kFALSE;
}
*/

// ------------------------------------------------------------------------------
//
// Read enviroment 
//
Int_t MContinueList::ReadEnv(const TEnv &env, TString prefix, Bool_t print)
{
    //
    // try to get class from root environment
    //
    const TString cname("MF");
    TClass *cls = gROOT->GetClass(cname);
    Int_t rc = 0;
    if (!cls)
        rc =1;
    else
    {
        if (!cls->Property())
            rc = 5;
        if (!cls->Size())
            rc = 4;
        if (!cls->IsLoaded())
            rc = 3;
        if (!cls->HasDefaultConstructor())
            rc = 2;
    }

    if (rc)
    {
	*fLog << err << dbginf << "Cannot create new instance of class '" << cname << "': ";
        switch (rc)
        {
        case 1:
            *fLog << "gROOT->GetClass() returned NULL." << endl;
            return kFALSE;
        case 2:
            *fLog << "no default constructor." << endl;
            return kFALSE;
        case 3:
            *fLog << "not loaded." << endl;
            return kFALSE;
        case 4:
            *fLog << "zero size." << endl;
            return kFALSE;
        case 5:
            *fLog << "no property." << endl;
            return kFALSE;
        }
    }

    if (!cls->InheritsFrom(MTask::Class()))
    {
        *fLog << " - Class doesn't inherit from MTask." << endl;
        return kFALSE;
    }

    //
    // create the parameter container of the the given class type
    //
    MF *pcont = (MF *)cls->New();
    if (!pcont)
    {
        *fLog << " - Class has no default constructor." << endl;
        *fLog << " - An abstract member functions of a base class is not overwritten." << endl;
        return kFALSE;
    }

    //
    // Set the name of the container
    //
    Int_t i = 0;
    TString oname = (prefix.CompareTo("")!=0) ? prefix.Data() : this->GetName();
    oname += ".Continue";
    oname += i;
    pcont->SetName(oname);

    //fFilters->AddToList(pcont);
    cout << oname << endl;
    while(pcont->ReadEnv(env,oname,kTRUE)){
	    *fLog << inf << "Setting enviroment for ";
	    *fLog << pcont->GetName() << " in " << fName << endl;
	    fFilters->AddToList(pcont);

	    oname = prefix + ".Continue";
	    //oname = "Continue";
	    oname += (++i);
	    pcont = (MF *)cls->New();
	    pcont->SetName(oname);
    }

    // Delete last created filter because it is empty
    delete pcont;

    return kTRUE;
}


// --------------------------------------------------------------------
//
// Invert the MContinueList condition but BE CAREFULL!!!!
// This invert the global condition as a single condition, if you
// want to invert each condition of the MContinueList use
// SetEachInverted() instead
//
void MContinueList::SetInverted(Bool_t inv)
{
    fFilters->SetInverted();
}



/*
TObject *MContinueList::FindObject(const TObject *obj)
{
    cout << "Inside " << this->GetName() << " object " << obj->GetName() << endl;
    cout << fFilters->FindObject(obj) << endl;
    return fFilters->FindObject(obj);
}


TObject *MContinueList::FindObject(const char *name)
{
    cout << "Inside " << this->GetName() << " object " << name << endl;
    cout << fFilters->FindObject(name) << endl;
    return fFilters->FindObject(name);
}
*/


